package com.fr.plugin.third.party.jsdjche.data;

import com.fanruan.api.log.LogKit;
import com.fanruan.api.util.StringKit;
import com.fr.base.TableData;
import com.fr.data.AbstractDataModel;
import com.fr.general.data.TableDataException;
import com.fr.json.JSONArray;
import com.fr.json.JSONObject;
import com.fr.plugin.context.PluginContexts;
import com.fr.plugin.third.party.jsdjche.Utils;
import com.fr.plugin.third.party.jsdjche.config.CustomDataConfig;
import com.fr.third.org.apache.http.impl.client.CloseableHttpClient;
import com.fr.utils.Base64;

import java.util.ArrayList;
import java.util.List;

public class CustomHttpDataModel extends AbstractDataModel {
    private static String[] COLUMN_NAMES = {"projectscadacode", "projectname", "servicestarttime", "serviceendtime"};
    private int rowCount = TableData.RESULT_ALL;
    private DatasetData datas = new DatasetData();

    public CustomHttpDataModel(int count) {
        this.rowCount = count;
        if (this.rowCount == 0) {
            return;
        }
        queryData();
    }

    @Override
    public int getColumnCount() throws TableDataException {
        return COLUMN_NAMES.length;
    }

    @Override
    public String getColumnName(int i) throws TableDataException {
        return COLUMN_NAMES[i];
    }

    @Override
    public boolean hasRow(int rowIndex) throws TableDataException {
        int count = getRowCount();
        return rowIndex < count;
    }

    @Override
    public int getRowCount() throws TableDataException {
        if (this.datas == null) {
            return 0;
        }
        List<List<Object>> values = this.datas.getValues();
        if (values == null) {
            return 0;
        }
        int count = values.size();
        return count;
    }

    @Override
    public Object getValueAt(int rowIndex, int columnIndex) throws TableDataException {
        if (this.datas == null) {
            return "";
        }
        List<List<Object>> values = this.datas.getValues();
        if ((values == null) || (values.size() <= rowIndex)) {
            return "";
        }
        List<Object> rowValues = values.get(rowIndex);
        if ((rowValues == null) || (rowValues.size() <= columnIndex)) {
            return "";
        }
        return rowValues.get(columnIndex);
    }

    @Override
    public void release() throws Exception {
        this.datas = null;
    }

    /**
     * 查询数据
     */
    private void queryData() {
        try {
            createDatas();
        } catch (Exception e) {
            LogKit.error("自定义数据集获取信息,请求出错," + e.getMessage() + "," + e.getCause(), e);
        }
    }

    private void createDatas() throws Exception {
        String userSyncUrl = CustomDataConfig.getInstance().getDataSyncUrl();
        String userSyncUsername = CustomDataConfig.getInstance().getDataUsername();
        String userSyncPassword = CustomDataConfig.getInstance().getDataPassword();
        if (StringKit.isEmpty(userSyncUrl) || StringKit.isEmpty(userSyncUsername) || StringKit.isEmpty(userSyncPassword)) {
            return;
        }

        CloseableHttpClient httpClient = Utils.createHttpClient(userSyncUrl);
        String authValue = userSyncUsername + ":" + userSyncPassword;
        String auth = "Basic " + Base64.getEncoder().encodeToString(authValue.getBytes("utf-8"));
        String content = "";
        LogKit.info("自定义数据集,请求数据..");

        //添加认证
        if (!PluginContexts.currentContext().isAvailable()) {
            LogKit.error("自定义数据集插件试用过期, 请购买许可证");
            return;
        }
        content = Utils.createHttpPostContent(httpClient, userSyncUrl,"{}", auth,"application/json");
        httpClient.close();
        if (StringKit.isEmpty(content)) {
            return;
        }
        LogKit.info("自定义数据集,json内容:\n" + content);
        addDatas(content);
    }

    private void addDatas(String content) throws TableDataException {
        if (StringKit.isEmpty(content)) {
            return;
        }
        JSONObject resJson = new JSONObject(content);
        int code = resJson.getInt("code");
        if (code != 1) {
            return;
        }

        JSONArray dataJsons = resJson.getJSONArray("data");
        if ((dataJsons == null) || (dataJsons.size() <= 0)) {
            return;
        }
        String projectScadaCode = "", projectName = "", serviceStartTime = "", serviceEndTime = "";
        JSONObject dataJson;
        int tempCount = 0;
        for (int i = 0, max = dataJsons.size() - 1; i <= max; i++) {
            if ((this.rowCount >= 1) && (tempCount >= this.rowCount)) {
                return;
            }
            dataJson = dataJsons.getJSONObject(i);
            projectScadaCode = dataJson.getString("projectScadaCode", "");
            projectName = dataJson.getString("projectName", "");
            serviceStartTime = dataJson.getString("serviceStartTime", "");
            serviceEndTime = dataJson.getString("serviceEndTime", "");
            addRowDatas(projectScadaCode, projectName, serviceStartTime, serviceEndTime);
            tempCount++;
        }
    }

    private void addRowDatas(String projectScadaCode, String projectName, String serviceStartTime, String serviceEndTime) {
        List<Object> rowDatas = new ArrayList<>();
        rowDatas.add(projectScadaCode);
        rowDatas.add(projectName);
        rowDatas.add(serviceStartTime);
        rowDatas.add(serviceEndTime);
        List<List<Object>> values = this.datas.getValues();
        values.add(rowDatas);
    }
}
