!(function () {
    var Model = BI.inherit(Fix.Model, {
        _init: function () {
            this.service = BI.Services.getService("dec.service.data.set");
        },
        context: ["dataSetName", "ableSave"],
        state: function () {
            var val = this.options.value.datasetData || {};
            return {
                config: val.config || "",
                parameters: val.parameters || []
            };
        },
        childContext: ["previewAble", "previewedDataSet"],
        watch: {
            previewAble: function (v) {
                this.setAbleSave(v);
            }
        },
        computed: {
            paramHeader: function () {
                //定义参数列表表头
                var map = BI.map(BI.Constants.getConstant("dec.constants.data.set.sql.header"), function (i, v) {
                    return BI.extend({
                        textAlign: "left",
                        height: 30,
                        hgap: 10
                    }, v);
                });
                map.push({
                    textAlign: "left",
                    height: 30,
                    hgap: 10,
                    text: ""
                });
                return [map];
            },
            paramItems: function () {
                var self = this;
                return BI.map(this.model.parameters, function (i, param) {
                    return [{
                        type: "bi.text_editor",
                        value: param.name,
                        textAlign: "left",
                        height: 30,
                        lgap: 10,
                        listeners: [{
                            eventName: BI.TextEditor.EVENT_CHANGE,
                            action: function () {
                                self.setParamName(i, this.getValue());
                            }
                        }]
                    }, {
                        type: "bi.icon_text_value_combo",
                        cls: "field-type-change",
                        height: 30,
                        items: BI.Constants.getConstant("dec.constants.data.set.sql.field"),
                        value: param.type,
                        listeners: [{
                            eventName: BI.IconTextValueCombo.EVENT_CHANGE,
                            action: function () {
                                self.setParamType(i, this.getValue()[0]);
                            }
                        }]
                    }, self.service.createParameterValueItem(param, function (val) {
                        self.setParamValue(i, val);
                    }), {
                        type: "bi.icon_button",
                        cls: "default-delete-font",
                        height: 30,
                        lgap: 10,
                        title: BI.i18nText("Dec-Basic_Delete"),
                        handler: function () {
                            self.deleteParameter(i);
                        }
                    }];
                });
            },
            previewAble: function () {
                return  true;
            },
            previewedDataSet: function () {
                return {
                    datasetType: DecCst.DataSet.Type.CUSTOM_DATA,
                    datasetName: this.model.dataSetName,
                    datasetData: {
                        parameters:[]
                    }
                };
            }
        },
        actions: {
            initPage: function (cb) {
                var self = this;
                cb();
            },
            setConfig: function (v) {
                this.model.config = v;
            },
            refreshParam: function () {
                var self = this;
                Dec.Utils.getDataSetParameters(this.model.previewedDataSet, function (res) {
                    self.model.parameters = self.service.getParameters(res.data, self.model.parameters);
                });
            },
            addParamter: function (v) {
                this.model.parameters.push(v);
            },
            deleteParameter: function (index) {
                this.model.parameters.splice(index, 1);
            },
            setParamName: function (index, name) {
                this.model.parameters[index].name = name;
            },
            setParamType: function (index, type) {
                if (this.model.parameters[index].type !== type) {
                    this.model.parameters[index].type = type;
                    this.model.parameters[index].value = this.service.getDefaultValueByType(type);
                    this.model.parameters.splice(0, 0);
                }
            },
            setParamValue: function (index, val) {
                this.model.parameters[index].value = val;
            },
            setAbleSave: function (v) {
                this.model.ableSave = v;
            }
        }
    });
    BI.model("dec.model.data.set.type.demo.data", Model);


    var DemoData = BI.inherit(BI.Widget, {
        props: {
            baseCls: "dec-data-set-sql",
            $testId: "dec-data-set-sql",
            value: {
                datasetData: {}
            }
        },
        _store: function () {
            return BI.Models.getModel("dec.model.data.set.type.demo.data", this.options);
        },
        watch: {
             config: function (v) {
                 this.configEditor.setValue(v);
             },
             "config || parameters.**": function () {
                 this.previewPane.resetPreview();
             },
             parameters: function () {
                 this.paramSettingPane.populate(this.model.paramItems, this.model.paramHeader);
             }
        },
        beforeInit: function (cb) {
            this.store.initPage(cb);
        },
        mounted: function () {
            // 在render方法里的子组件被渲染到页面之后的钩子函数
            this.previewPane.resetPreview();
            this.store.setAbleSave(true);
        },
        render: function () {
            var self = this;
            return {
                type: "bi.htape",
                items: [
                    //左侧的配置界面
                    /*{
                        el: {
                            type: "bi.vtape",
                            items: [
                                {
                                    height: 24,
                                    type: "bi.label",
                                    cls: "dec-font-weight-bold",
                                    text: "Config",
                                    title: "Config",
                                    textAlign: "left",
                                },
                                {
                                    height: 220,
                                    type: "bi.textarea_editor",
                                    cls: "bi-border",
                                    value: this.model.config,
                                    ref: function (_ref) {
                                        self.configEditor = _ref;
                                    },
                                    listeners: [{
                                        eventName: BI.FormulaEditor.EVENT_CHANGE,
                                        action: function () {
                                            self.store.setConfig(this.getValue());
                                        }
                                    }]
                                },
                                //下面是参数面板
                                {
                                    type: "bi.left_right_vertical_adapt",
                                    height: 24,
                                    items: {
                                        left: [{
                                            type: "bi.label",
                                            cls: "dec-font-weight-bold",
                                            text: BI.i18nText("Dec-Basic_Parameter_Setting")
                                        }],
                                        right: [{
                                            type: "bi.icon_button",
                                            cls: "add-keyword-font",
                                            level: "ignore",
                                            lgap: 10,
                                            handler: function () {
                                                self.store.addParamter({
                                                    type: "String",
                                                    name: "",
                                                    value: ""
                                                });
                                            }
                                        }, {
                                            type: "bi.icon_button",
                                            cls: "refresh-font",
                                            lgap: 10,
                                            level: "ignore",
                                            handler: function () {
                                                self.store.refreshParam();
                                            }
                                        }]
                                    }
                                }, {
                                    type: "bi.components.style_table",
                                    cls: "param-table",
                                    width: 420,
                                    columnSize: [130, 120, 120, ""],
                                    items: this.model.paramItems,
                                    header: this.model.paramHeader,
                                    ref: function () {
                                        self.paramSettingPane = this;
                                    }
                                }
                            ],
                            lgap: 10,
                            vgap: 15
                        },
                        width: 410,
                        lgap: 10,
                        vgap: 15
                    },*/
                    //右侧的预览界面
                    {
                        el: {
                            type: "bi.absolute",
                            cls: "bi-sortable-holder",
                            items: [{
                                el: {
                                    type: "dec.data.set.preview",
                                    listeners: [{
                                        eventName: "EVENT_CHANGE",
                                        action: function () {
                                            //self.store.refreshParam();
                                            self.previewPane.previewData();
                                        }
                                    }],
                                    ref: function (_ref) {
                                        self.previewPane = _ref;
                                    }
                                }, top: 10, right: 10, bottom: 10, left: 10
                            }]
                        },
                        hgap: 10,
                        vgap: 15
                    }
                ]
            };
        },
        getValue: function () {
            return {
                config: this.model.config,
                parameters: this.model.parameters
            };
        }
    });
    BI.shortcut("dec.data.set.type.custom.data", DemoData);

    $.extend(DecCst.DataSet.Type, {CUSTOM_DATA: "custom_data"});
    BI.config("dec.provider.data.set", function (provider) {
        provider.registerDataSetType({
            value: DecCst.DataSet.Type.CUSTOM_DATA,
            text: "自定义数据集",
            cardType: "dec.data.set.type.custom.data",
            iconCls: "management-maintenance-cloud-font"
        });
    });
})();